package org.jeecg.modules.prowork.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.google.common.collect.Lists;
import io.swagger.annotations.ApiModel;
import org.jeecg.common.dto.prowork.ProWorkLogicAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkLogicContentAddReqDTO;
import org.jeecg.common.dto.prowork.ProWorkLogicContentListRespDTO;
import org.jeecg.common.dto.prowork.ProWorkLogicRespDTO;
import org.jeecg.common.exception.JeecgBootException;
import org.jeecg.modules.prowork.entity.ProWorkLogic;
import org.jeecg.modules.prowork.entity.ProWorkLogicContent;
import org.jeecg.modules.prowork.mapper.ProWorkLogicMapper;
import org.jeecg.modules.prowork.service.ProWorkLogicContentService;
import org.jeecg.modules.prowork.service.ProWorkLogicService;
import org.jeecg.modules.system.service.ISysSerialPatternService;
import org.jeecg.modules.system.vo.CallResult;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.util.CollectionUtils;

import java.util.List;

/**
 * @Description: 日志(实施，开发，服务)
 * @author： jeecg-boot
 * @date：   2021-03-02
 * @version： V1.0
 */
@Service
public class ProWorkLogicServiceImpl extends ServiceImpl<ProWorkLogicMapper, ProWorkLogic> implements ProWorkLogicService {
    @Autowired
    private ProWorkLogicContentService proWorkLogicContentService;
    @Autowired
    private ISysSerialPatternService sysSerialPatternService;

    /**
     * @desc 新增
     */
    @Override
    public void add(ProWorkLogicAddReqDTO reqDTO) throws Exception {
        if(CollectionUtils.isEmpty(reqDTO.getDetailList())){
            throw new JeecgBootException("子表不能为空");
        }
        ProWorkLogic proWorkLogic = new ProWorkLogic(reqDTO);
        // 设置编码
        CallResult<String> nextSerial = sysSerialPatternService.getNextSerial(proWorkLogic, "pro_work_logic", "billcode", true);
        if(!nextSerial.isSucceed()){
            throw new RuntimeException("获取编号失败");
        }
        proWorkLogic.setBillcode(nextSerial.getContent());
        save(proWorkLogic);
        //添加子表
        for(ProWorkLogicContentAddReqDTO detail : reqDTO.getDetailList()){
            detail.setLogicId(proWorkLogic.getId());
            ProWorkLogicContent proWorkLogicContent =  new ProWorkLogicContent(detail);
            proWorkLogicContentService.save(proWorkLogicContent);
        }
    }

    /**
     * @desc 编辑
     */
    @Override
    public boolean edit(ProWorkLogicAddReqDTO reqDTO) {
        ProWorkLogic proWorkLogic = new ProWorkLogic(reqDTO);
        updateById(proWorkLogic);
        //删除子表
        deletChildren(reqDTO.getId());
        //添加子表
        for(ProWorkLogicContentAddReqDTO detail : reqDTO.getDetailList()){
            detail.setLogicId(proWorkLogic.getId());
            ProWorkLogicContent proWorkLogicContent =  new ProWorkLogicContent(detail);
            proWorkLogicContentService.save(proWorkLogicContent);
        }
        return true;
    }

    /**
     * @desc 通过id删除
     */
    @Override
    public boolean dropById(ProWorkLogic proWorkLogic) {
        proWorkLogic.setDelFlag("1");
        updateById(proWorkLogic);
        //删除子表
        deletChildren(proWorkLogic.getId());
        return true;
    }

    /**
     * @desc 通过id查询日志（实施，开发，服务）
     */
    @Override
    public ProWorkLogicRespDTO queryById(String id) {
        ProWorkLogic proWorkLogic = getById(id);
        ProWorkLogicRespDTO respDTO = new ProWorkLogicRespDTO(proWorkLogic);
        //查询子表
        QueryWrapper<ProWorkLogicContent> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("logic_id", id);
        List<ProWorkLogicContent> list = proWorkLogicContentService.list(queryWrapper);
        List<ProWorkLogicContentListRespDTO> contentListRespDTOS = Lists.newArrayList();
        for(ProWorkLogicContent proWorkLogicContent : list){
            contentListRespDTOS.add(new ProWorkLogicContentListRespDTO(proWorkLogicContent));
        }
        respDTO.setDetailList(contentListRespDTOS);
        return respDTO;
    }



    /**
     * @desc 删除子表
     */
    private void deletChildren(String id) {
        QueryWrapper<ProWorkLogicContent> queryWrapper = new QueryWrapper();
        queryWrapper.eq("del_flag", "0");
        queryWrapper.eq("logic_id", id);
        proWorkLogicContentService.update(new ProWorkLogicContent("1"), queryWrapper);
    }
}
